import styled from 'styled-components';
import { motion } from 'framer-motion';
import { produce } from 'immer';
import { isEmpty } from 'lodash-es';
import theme from '../../../../theme';
import { Container, Stack } from '../../../layout';
import Divider from '../../atoms/Divider/Divider';
import Paragraph, { ParagraphProps } from '../../atoms/Paragraph/Paragraph';
import Title, { TitleProps } from '../../atoms/Title/Title';
import Button, { ButtonProps } from '../../molecules/Button/Button';

interface FooterItem {
  titleProps: ParagraphProps;
  descriptionProps: ParagraphProps;
}

export interface Footer2Props {
  titleProps: TitleProps;
  paragraphProps: ParagraphProps;
  buttonsProps: ButtonProps[];
  footerItems1: FooterItem[];
  footerItems2: FooterItem[];
  footerItems3: FooterItem[];
  footerItems4: FooterItem[];
  copyrightProps: ParagraphProps;
  backgroundImage?: string;
  backgroundColor?: string;
  isMobile?: boolean;
  isCenter?: boolean;
  paddingY?: number;
}

export default function Footer2(props: Footer2Props) {
  const { titleProps, paragraphProps, buttonsProps, footerItems1, footerItems2, footerItems3, footerItems4, copyrightProps, backgroundImage, backgroundColor, isMobile, isCenter, paddingY } = props;

  return (
    <Container isMobile={isMobile} direction="column" isCenter={isCenter || isMobile} backgroundColor={backgroundColor} backgroundImage={backgroundImage} padding={`${paddingY}px 0`}>
      {titleProps.isVisible && (
        <Box margin="0 0 8px">
          <Title {...titleProps} isMobile={isMobile} />
        </Box>
      )}
      {paragraphProps.isVisible && (
        <Box margin={isMobile ? '0 0 32px' : '0 0 64px'}>
          <Paragraph {...paragraphProps} isMobile={isMobile} textAlign={isMobile ? 'center' : 'left'} />
        </Box>
      )}
      {!isMobile && (
        <Stack direction="column" gap={12}>
          {!isEmpty(footerItems1) && <FooterItems items={footerItems1} />}
          {!isEmpty(footerItems2) && <FooterItems items={footerItems2} />}
          {!isEmpty(footerItems3) && <FooterItems items={footerItems3} />}
          {!isEmpty(footerItems4) && <FooterItems items={footerItems4} />}
        </Stack>
      )}
      {isMobile && (
        <Stack direction="column" gap={12}>
          {!isEmpty(footerItems1) && <MobileFooterItems items={footerItems1} />}
          {!isEmpty(footerItems2) && <MobileFooterItems items={footerItems2} />}
          {!isEmpty(footerItems3) && <MobileFooterItems items={footerItems3} />}
          {!isEmpty(footerItems4) && <MobileFooterItems items={footerItems4} />}
        </Stack>
      )}
      <Box width="100%">
        <Divider margin={isMobile ? '32px 0' : '40px 0'} color={theme.colors.gray100} isFullWidth={true} />
      </Box>
      <Stack direction={isMobile ? 'column' : 'row'} justifyContent="space-between" gap={isMobile ? 12 : 0}>
        {!isEmpty(buttonsProps) && (
          <Stack gap={12} direction={'row'} flexWrap="wrap">
            {buttonsProps?.map((button, index) => {
              return <Button key={index} {...button} />;
            })}
          </Stack>
        )}
        <Paragraph {...copyrightProps} whiteSpace="nowrap" isMobile={isMobile} />
      </Stack>
    </Container>
  );
}

function FooterItems({ items }: { items: FooterItem[] }) {
  return (
    <Stack gap={12} direction="row" justifyContent="flex-start" alignItems="center">
      {items?.map((item, index) => {
        const isLastIndex = items?.length - 1 === index;
        return (
          <>
            <Stack gap={10} direction="row" alignItems="center" width="auto">
              <Paragraph {...item.titleProps} whiteSpace="nowrap" />
              <Paragraph {...item.descriptionProps} />
            </Stack>
            {!isLastIndex && <Divider color={theme.colors.gray100} length={20} direction="vertical" />}
          </>
        );
      })}
    </Stack>
  );
}

function MobileFooterItems({ items }: { items: FooterItem[] }) {
  return (
    <Stack gap={12} direction="column" justifyContent="flex-start" alignItems="flex-start">
      {items?.map((item, index) => {
        return (
          <>
            <Stack gap={10} direction="row" alignItems="flex-start" width="auto">
              <Paragraph {...item.titleProps} whiteSpace="nowrap" isMobile={true} />
              <Paragraph {...item.descriptionProps} isMobile={true} />
            </Stack>
          </>
        );
      })}
    </Stack>
  );
}

Footer2.defaultProps = {
  titleProps: produce(Title.defaultProps, draft => {
    draft.content = 'Title 01 타이틀 01\nTitle 02 타이틀 02';
    draft.size = 'lg';
    draft.highlight.color = theme.colors.gray500;
  }),
  paragraphProps: produce(Paragraph.defaultProps, draft => {
    draft.content = 'Body 입력 Body 입력 무엇을 구하지 꽃이 무엇이 천고에 천자만홍이 황금시대다. 피는 대한 따뜻한 꽃 가슴이 봄바람이다.속에서 밥을 끓는 속잎나고.';
    draft.size = 'lg';
    draft.color = theme.colors.gray500;
  }),
  footerItems1: [
    {
      titleProps: produce(Paragraph.defaultProps, draft => {
        draft.content = '코드인디자인 CodeInDesign';
        draft.fontWeight = 'bold';
      }),
      descriptionProps: produce(Paragraph.defaultProps, draft => {
        draft.content = '';
        draft.isVisible = false;
      }),
    },
  ],
  footerItems2: [
    {
      titleProps: produce(Paragraph.defaultProps, draft => {
        draft.content = 'E-mail';
        draft.fontWeight = 'bold';
      }),
      descriptionProps: produce(Paragraph.defaultProps, draft => {
        draft.content = 'codeindesign.official@gmail.com';
      }),
    },
  ],
  footerItems3: [
    {
      titleProps: produce(Paragraph.defaultProps, draft => {
        draft.content = '주소';
        draft.fontWeight = 'bold';
      }),
      descriptionProps: produce(Paragraph.defaultProps, draft => {
        draft.content = '서울특별시 송파구 잠실동 204-13 4층';
      }),
    },
  ],
  footerItems4: [
    {
      titleProps: produce(Paragraph.defaultProps, draft => {
        draft.content = '주소';
        draft.fontWeight = 'bold';
      }),
      descriptionProps: produce(Paragraph.defaultProps, draft => {
        draft.content = '서울특별시 송파구 잠실동 204-13 4층';
      }),
    },
  ],
  copyrightProps: produce(Paragraph.defaultProps, draft => {
    draft.content = 'All Rights Reserved.';
    draft.size = 'md';
    draft.color = theme.colors.gray300;
  }),
  buttonsProps: Array(1).fill(
    produce(Button.defaultProps, draft => {
      draft.size = 'sm';
      draft.buttonType = 'link';
      draft.color = theme.colors.black;
    }),
  ),
  backgroundImage: 'https://picsum.photos/id/347/2000',
  backgroundColor: 'transparent',
  isMobile: false,
  isCenter: true,
  paddingY: 100,
};

const Box = styled(motion.div)<any>`
  margin: ${props => props.margin};
  width: ${props => props.width};
  background-color: ${props => props.backgroundColor};
  background-image: url('${props => props.backgroundImage}');
  background-size: cover;
`;

const ButtonBox = styled(motion.div)<any>`
  flex: 1;
  width: 100%;
`;

const FeatureImage = styled(motion.img)<any>`
  width: 100%;
  height: 600px;
  object-fit: cover;
  border-radius: 20px;
  overflow: hidden;
`;
